/*
 * Mach Operating System
 * Copyright (c) 1992, 1991 Carnegie Mellon University
 * Copyright (c) 1991 OMRON Corporation
 * All Rights Reserved.
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON AND OMRON ALLOW FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON AND OMRON DISCLAIM ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie the
 * rights to redistribute these changes.
 */
/*
 * Amiga kernel console code
 *
 * Contributors:
 *      Bryan Ford
 *	Niklas Hallqvist
 */

#include <mach/boolean.h>
#include <machine/machparam.h>
#include <machine/kd.h>
#include <machine/uart.h>
#include <device/cirbuf.h>

#define CONS_BUFSIZE 256

static struct cirbuf phys_cons_buf;
struct cirbuf *cons_buf = &phys_cons_buf;
static enum { UNDEFINED, RAW, TTY } state = UNDEFINED;

/* Called from init.c */
void
cninit (void)
{
  kdinit ();
  utinit ();
  state = RAW;
  /* We can't use cb_alloc here because kalloc isn't initialized yet */
#if 0
  cb_alloc (cons_buf, CONS_BUFSIZE);
#else
  {
    register struct cirbuf *cb = cons_buf;
    register int buf_size = CONS_BUFSIZE;
    static char buf[CONS_BUFSIZE];

    cb->c_start = buf;
    cb->c_end = buf + buf_size;
    cb->c_cf = buf;
    cb->c_cl = buf;
    cb->c_cc = 0;
    cb->c_hog = buf_size - 1;
  }
#endif
}

/* Send a character from the kernel to both the screen and the serial port
 */
void
cnputc (u_char c)
{
  if(state != UNDEFINED)
    {
      if (c == '\n')
        cnputc('\r');
      if (state == RAW)
	{
	  ut_cnputc (c);
	  kd_cnputc (c);
	}
      else
	{
	  ttyoutput (c, &kd_tty);
	  ttyoutput (c, &uart_tty);
	}
    }
}


/* Get a key from either the keyboard or the serial port */
int
cngetc (void)
{
  int c = -1;

  switch (state)
    {
    case UNDEFINED:
      break;
    case RAW:
      while (c < 0)
	{
	  int s = spltty ();
	  c = getc (cons_buf);
	  splx (s);
	}
      break;
    default:
      panic ("cngetc: state not implemented\n");
    }

  return c;
}
